@@ -74,6 +74,16 @@
          (ftype (function ((unsigned-byte 32) (unsigned-byte 32)) (unsigned-byte 32)) mod32+))
 (defun mod32+ (a b)
   (declare (type (unsigned-byte 32) a b))
+  #+ecl
+  (ffi:c-inline (a b)
+                (:uint32-t :uint32-t)
+                :uint32-t
+                "#0 + #1"
+                :one-liner t
+                :side-effects nil)
+  #+(and ccl x86-64)
+  (ccl::mod32+ a b)
+  #-(or ecl (and ccl x86-64))
   (ldb (byte 32 0) (+ a b)))
 
 #+cmu
@@ -90,6 +100,16 @@
 
 (defun mod32- (a b)
   (declare (type (unsigned-byte 32) a b))
+  #+ecl
+  (ffi:c-inline (a b)
+                (:uint32-t :uint32-t)
+                :uint32-t
+                "#0 - #1"
+                :one-liner t
+                :side-effects nil)
+  #+(and ccl x86-64)
+  (ccl::mod32- a b)
+  #-(or ecl (and ccl x86-64))
   (ldb (byte 32 0) (- a b)))
 
 #+cmu
@@ -106,6 +126,16 @@
 
 (defun mod32* (a b)
   (declare (type (unsigned-byte 32) a b))
+  #+ecl
+  (ffi:c-inline (a b)
+                (:uint32-t :uint32-t)
+                :uint32-t
+                "#0 * #1"
+                :one-liner t
+                :side-effects nil)
+  #+(and ccl x86-64)
+  (ccl::mod32* a b)
+  #-(or ecl (and ccl x86-64))
   (ldb (byte 32 0) (* a b)))
 
 #+cmu
@@ -120,8 +150,18 @@
          (ftype (function ((unsigned-byte 32) (integer -31 31)) (unsigned-byte 32)) mod32ash))
 
 (defun mod32ash (num count)
-  (declare (type (unsigned-byte 32) num))
-  (declare (type (integer -31 31) count))
+  (declare (type (unsigned-byte 32) num)
+           (type (integer -31 31) count))
+  #+ecl
+  (ffi:c-inline (num count)
+                (:uint32-t :int8-t)
+                :uint32-t
+                "(#1 > 0) ? (#0 << #1) : (#0 >> -#1)"
+                :one-liner t
+                :side-effects nil)
+  #+(and ccl x86-64)
+  (ccl::mod32ash num count)
+  #-(or ecl (and ccl x86-64))
   (ldb (byte 32 0) (ash num count)))
 
 #+sbcl
@@ -134,6 +174,17 @@
          (ftype (function ((unsigned-byte 32)) (unsigned-byte 32)) mod32lognot))
 
 (defun mod32lognot (num)
+  (declare (type (unsigned-byte 32) num))
+  #+ecl
+  (ffi:c-inline (num)
+                (:uint32-t)
+                :uint32-t
+                "~#0"
+                :one-liner t
+                :side-effects nil)
+  #+(and ccl x86-64)
+  (ccl::mod32lognot num)
+  #-(or ecl (and ccl x86-64))
   (ldb (byte 32 0) (lognot num)))
 
 #+sbcl
@@ -145,26 +196,55 @@
 
 (defun rol32 (a s)
   (declare (type (unsigned-byte 32) a) (type (integer 0 32) s))
+  #+(and ccl x86-64)
+  (ccl::rol32 a s)
   #+cmu
   (kernel:32bit-logical-or #+little-endian (kernel:shift-towards-end a s)
                            #+big-endian (kernel:shift-towards-start a s)
                            (ash a (- s 32)))
+  #+ecl
+  (ffi:c-inline (a s)
+                (:uint32-t :uint8-t)
+                :uint32-t
+                "(#0 << #1) | (#0 >> (32 - #1))"
+                :one-liner t
+                :side-effects nil)
   #+sbcl
   (sb-rotate-byte:rotate-byte s (byte 32 0) a)
-  #-(or sbcl cmu)
+  #-(or (and ccl x86-64) cmu ecl sbcl)
   (logior (ldb (byte 32 0) (ash a s)) (ash a (- s 32))))
 
 (defun ror32 (a s)
   (declare (type (unsigned-byte 32) a) (type (integer 0 32) s))
+  #+(and ccl x86-64)
+  (ccl::ror32 a s)
+  #+ecl
+  (ffi:c-inline (a s)
+                (:uint32-t :uint8-t)
+                :uint32-t
+                "(#0 << (32 - #1)) | (#0 >> #1)"
+                :one-liner t
+                :side-effects nil)
   #+sbcl
   (sb-rotate-byte:rotate-byte (- s) (byte 32 0) a)
-  #-sbcl
+  #-(or (and ccl x86-64) ecl sbcl)
   (rol32 a (- 32 s)))
 
 (declaim #+ironclad-fast-mod64-arithmetic (inline mod64+ mod64- mod64*)
          (ftype (function ((unsigned-byte 64) (unsigned-byte 64)) (unsigned-byte 64)) mod64+))
+
 (defun mod64+ (a b)
   (declare (type (unsigned-byte 64) a b))
+  #+ecl
+  (ffi:c-inline (a b)
+                (:uint64-t :uint64-t)
+                :uint64-t
+                "#0 + #1"
+                :one-liner t
+                :side-effects nil)
+  #+(and ccl ironclad-fast-mod64-arithmetic)
+  (ccl::mod64+ a b)
+  #-(or ecl (and ccl ironclad-fast-mod64-arithmetic))
   (ldb (byte 64 0) (+ a b)))
 
 #+sbcl
@@ -173,6 +253,16 @@
 
 (defun mod64- (a b)
   (declare (type (unsigned-byte 64) a b))
+  #+ecl
+  (ffi:c-inline (a b)
+                (:uint64-t :uint64-t)
+                :uint64-t
+                "#0 - #1"
+                :one-liner t
+                :side-effects nil)
+  #+(and ccl ironclad-fast-mod64-arithmetic)
+  (ccl::mod64- a b)
+  #-(or ecl (and ccl ironclad-fast-mod64-arithmetic))
   (ldb (byte 64 0) (- a b)))
 
 #+sbcl
@@ -181,21 +271,38 @@
 
 (defun mod64* (a b)
   (declare (type (unsigned-byte 64) a b))
+  #+ecl
+  (ffi:c-inline (a b)
+                (:uint64-t :uint64-t)
+                :uint64-t
+                "#0 * #1"
+                :one-liner t
+                :side-effects nil)
+  #+(and ccl ironclad-fast-mod64-arithmetic)
+  (ccl::mod64* a b)
+  #-(or ecl (and ccl ironclad-fast-mod64-arithmetic))
   (ldb (byte 64 0) (* a b)))
 
 #+sbcl
 (define-compiler-macro mod64* (a b)
   `(ldb (byte 64 0) (* ,a ,b)))
 
-(declaim #+ironclad-fast-mod64-arithmetic (inline rol64 ror64)
-         (ftype (function ((unsigned-byte 64) (unsigned-byte 6)) (unsigned-byte 64)) rol64 ror64))
-
 (declaim #+ironclad-fast-mod64-arithmetic (inline mod64ash)
          (ftype (function ((unsigned-byte 64) (integer -63 63)) (unsigned-byte 64)) mod64ash))
 
 (defun mod64ash (num count)
-  (declare (type (unsigned-byte 64) num))
-  (declare (type (integer -63 63) count))
+  (declare (type (unsigned-byte 64) num)
+           (type (integer -63 63) count))
+  #+ecl
+  (ffi:c-inline (num count)
+                (:uint64-t :int8-t)
+                :uint64-t
+                "(#1 > 0) ? (#0 << #1) : (#0 >> -#1)"
+                :one-liner t
+                :side-effects nil)
+  #+(and ccl ironclad-fast-mod64-arithmetic)
+  (ccl::mod64ash num count)
+  #-(or ecl (and ccl ironclad-fast-mod64-arithmetic))
   (ldb (byte 64 0) (ash num count)))
 
 #+sbcl
@@ -208,6 +315,17 @@
          (ftype (function ((unsigned-byte 64)) (unsigned-byte 64)) mod64lognot))
 
 (defun mod64lognot (num)
+  (declare (type (unsigned-byte 64) num))
+  #+ecl
+  (ffi:c-inline (num)
+                (:uint64-t)
+                :uint64-t
+                "~#0"
+                :one-liner t
+                :side-effects nil)
+  #+(and ccl ironclad-fast-mod64-arithmetic)
+  (ccl::mod64lognot num)
+  #-(or ecl (and ccl ironclad-fast-mod64-arithmetic))
   (ldb (byte 64 0) (lognot num)))
 
 #+sbcl
@@ -219,16 +337,38 @@
 
 (defun rol64 (a s)
   (declare (type (unsigned-byte 64) a) (type (integer 0 64) s))
+  #+ecl
+  (ffi:c-inline (a s)
+                (:uint64-t :uint8-t)
+                :uint64-t
+                "(#0 << #1) | (#0 >> (64 - #1))"
+                :one-liner t
+                :side-effects nil)
   #+(and sbcl ironclad-fast-mod64-arithmetic)
   (sb-rotate-byte:rotate-byte s (byte 64 0) a)
-  #-(and sbcl ironclad-fast-mod64-arithmetic)
+  #+(and ccl ironclad-fast-mod64-arithmetic)
+  (ccl::rol64 a s)
+  #-(or ecl
+        (and sbcl ironclad-fast-mod64-arithmetic)
+        (and ccl ironclad-fast-mod64-arithmetic))
   (logior (ldb (byte 64 0) (ash a s)) (ash a (- s 64))))
 
 (defun ror64 (a s)
   (declare (type (unsigned-byte 64) a) (type (integer 0 64) s))
+  #+ecl
+  (ffi:c-inline (a s)
+                (:uint64-t :uint8-t)
+                :uint64-t
+                "(#0 << (64 - #1)) | (#0 >> #1)"
+                :one-liner t
+                :side-effects nil)
   #+(and sbcl ironclad-fast-mod64-arithmetic)
   (sb-rotate-byte:rotate-byte (- s) (byte 64 0) a)
-  #-(and sbcl ironclad-fast-mod64-arithmetic)
+  #+(and ccl ironclad-fast-mod64-arithmetic)
+  (ccl::ror64 a s)
+  #-(or ecl
+        (and sbcl ironclad-fast-mod64-arithmetic)
+        (and ccl ironclad-fast-mod64-arithmetic))
   (rol64 a (- 64 s)))
 
 
@@ -395,7 +535,6 @@ behavior."
         do (setf (aref block i) (nibbles:ub64ref/be buffer j)))
   (values))
 
-(declaim (notinline xor-block))
 (defun xor-block (block-length input-block1 input-block2 input-block2-start
                                output-block output-block-start)
   (declare (type (simple-array (unsigned-byte 8) (*)) input-block1 input-block2 output-block))
@@ -434,12 +573,13 @@ behavior."
           (zerop (mod block-length sb-vm:n-word-bytes)))
      (let ((accessor (ecase sb-vm:n-word-bits
                        (32 'nibbles:ub32ref/le)
-                       (64 'nibbles:ub64ref/le))))
-       `(loop for i from 0 below ,block-length by ,sb-vm:n-word-bytes
-              do (setf (,accessor ,output-block (+ ,output-block-start i))
-                       (logxor (,accessor ,input-block1 i)
+                       (64 'nibbles:ub64ref/le)))
+           (i (gensym)))
+       `(loop for ,i from 0 below ,block-length by ,sb-vm:n-word-bytes
+              do (setf (,accessor ,output-block (+ ,output-block-start ,i))
+                       (logxor (,accessor ,input-block1 ,i)
                                (,accessor ,input-block2
-                                          (+ ,input-block2-start i)))))))
+                                          (+ ,input-block2-start ,i)))))))
     (t
      form)))
 
