@@ -14,9 +14,7 @@
 
 (defun update-digest-from-stream (digest stream &key buffer (start 0) end)
   (cond
-    ((let ((element-type (stream-element-type stream)))
-       (or (equal element-type '(unsigned-byte 8))
-           (equal element-type '(integer 0 255))))
+    ((subtypep (stream-element-type stream) '(unsigned-byte 8))
      (flet ((frob (read-buffer start end)
               (loop for last-updated = (read-sequence read-buffer stream
                                                       :start start :end end)
@@ -31,8 +29,9 @@
              (declare (dynamic-extent buffer))
              (frob buffer 0 +buffer-size+)))))
     (t
-     (error "Unsupported stream element-type ~S for stream ~S."
-            (stream-element-type stream) stream))))
+     (error 'ironclad-error
+            :format-control "Unsupported stream element-type ~S for stream ~S."
+            :format-arguments (list (stream-element-type stream) stream)))))
 
 ;;; Storing a length at the end of the hashed data is very common and
 ;;; can be a small bottleneck when generating lots of hashes over small
@@ -177,17 +176,18 @@
              (let ((digest-size ,(if single-digest-p
                                      (second (first specs))
                                      `(etypecase state
-                                        ,@(reverse specs)))))
+                                        ,@(reverse specs))))
+                   (state-copy (copy-digest state)))
                (etypecase digest
                  (simple-octet-vector
                   ;; verify that the buffer is large enough
                   (if (<= digest-size (- (length digest) digest-start))
-                      (,inner-fun-name state digest digest-start)
+                      (,inner-fun-name state-copy digest digest-start)
                       (error 'insufficient-buffer-space
                              :buffer digest :start digest-start
                              :length digest-size)))
-                 (cl:null
-                  (,inner-fun-name state
+                 (null
+                  (,inner-fun-name state-copy
                                    (make-array digest-size
                                                :element-type '(unsigned-byte 8))
                                    0))))))))))
@@ -365,7 +365,7 @@ The exact method is determined by the type of THING."))
 
 (defgeneric produce-digest (digester &key digest digest-start)
   (:documentation "Return the hash of the data processed by
-DIGESTER so far. This function modifies the internal state of DIGESTER.
+DIGESTER so far.
 
 If DIGEST is provided, the hash will be placed into DIGEST starting at
 DIGEST-START.  DIGEST must be a (SIMPLE-ARRAY (UNSIGNED-BYTE 8) (*)).
@@ -386,7 +386,7 @@ An error will be signaled if there is insufficient room in DIGEST."))
 (defun digest-supported-p (name)
   "Return T if the digest NAME is a valid digest name."
   (and (symbolp name)
-       (not (cl:null (digestp name)))))
+       (not (null (digestp name)))))
 
 (defgeneric digest-length (digest)
   (:documentation "Return the number of bytes in a digest generated by DIGEST."))
