@@ -5,7 +5,7 @@
 
 (defun quotationp (thing)
   (and (consp thing) (consp (rest thing))
-       (cl:null (cddr thing)) (eq (first thing) 'quote)))
+       (null (cddr thing)) (eq (first thing) 'quote)))
 
 (defun unquote (thing)
   (if (quotationp thing) (second thing) thing))
