@@ -6,7 +6,7 @@
 
 ;;; portability definitions
 
-#+cmu
+#+(or cmu abcl)
 (eval-when (:compile-toplevel :load-toplevel :execute)
   (require :gray-streams))
 
@@ -19,6 +19,10 @@
                    'excl:stream-write-string)
     (require "streamc.fasl")))
 
+#+ecl
+(eval-when (:compile-toplevel :load-toplevel :execute)
+  (gray::redefine-cl-functions))
+
 (eval-when (:compile-toplevel :load-toplevel :execute)
 (defvar *binary-input-stream-class*
   (quote
@@ -27,8 +31,10 @@
    #+openmcl gray:fundamental-binary-input-stream
    #+cmu ext:fundamental-binary-input-stream
    #+allegro excl:fundamental-binary-input-stream
-   #-(or lispworks sbcl openmcl cmu allegro)
-   (error "octet streams not supported in this implementation")))
+   #+abcl gray-streams:fundamental-binary-input-stream
+   #+(or ecl clisp) gray:fundamental-binary-input-stream
+   #-(or lispworks sbcl openmcl cmu allegro abcl ecl clisp)
+   (error 'ironclad-error :format-control "octet streams not supported in this implementation")))
 
 (defvar *binary-output-stream-class*
   (quote
@@ -37,8 +43,10 @@
    #+openmcl gray:fundamental-binary-output-stream
    #+cmu ext:fundamental-binary-output-stream
    #+allegro excl:fundamental-binary-output-stream
-   #-(or lispworks sbcl openmcl cmu allegro)
-   (error "octet streams not supported in this implementation")))
+   #+abcl gray-streams:fundamental-binary-output-stream
+   #+(or ecl clisp) gray:fundamental-binary-output-stream
+   #-(or lispworks sbcl openmcl cmu allegro abcl ecl clisp)
+   (error 'ironclad-error :format-control "octet streams not supported in this implementation")))
 
 ;;; FIXME: how to do CMUCL support for this?
 (defvar *stream-element-type-function*
@@ -48,8 +56,10 @@
    #+openmcl cl:stream-element-type
    #+cmu cl:stream-element-type
    #+allegro cl:stream-element-type
-   #-(or lispworks sbcl openmcl cmu allegro)
-   (error "octet streams not supported in this implementation")))
+   #+abcl cl:stream-element-type
+   #+(or ecl clisp) cl:stream-element-type
+   #-(or lispworks sbcl openmcl cmu allegro abcl ecl clisp)
+   (error 'ironclad-error :format-control "octet streams not supported in this implementation")))
 
 (defvar *stream-read-byte-function*
   (quote
@@ -58,8 +68,10 @@
    #+openmcl gray:stream-read-byte
    #+cmu ext:stream-read-byte
    #+allegro excl:stream-read-byte
-   #-(or lispworks sbcl openmcl cmu allegro)
-   (error "octet streams not supported in this implementation")))
+   #+abcl gray-streams:stream-read-byte
+   #+(or ecl clisp) gray:stream-read-byte
+   #-(or lispworks sbcl openmcl cmu allegro abcl ecl clisp)
+   (error 'ironclad-error :format-control "octet streams not supported in this implementation")))
 
 (defvar *stream-write-byte-function*
   (quote
@@ -68,8 +80,10 @@
    #+openmcl gray:stream-write-byte
    #+cmu ext:stream-write-byte
    #+allegro excl:stream-write-byte
-   #-(or lispworks sbcl openmcl cmu allegro)
-   (error "octet streams not supported in this implementation")))
+   #+abcl gray-streams:stream-write-byte
+   #+(or ecl clisp) gray:stream-write-byte
+   #-(or lispworks sbcl openmcl cmu allegro abcl ecl clisp)
+   (error 'ironclad-error :format-control "octet streams not supported in this implementation")))
 
 (defvar *stream-read-sequence-function*
   (quote
@@ -78,8 +92,10 @@
    #+openmcl ccl:stream-read-vector
    #+cmu ext:stream-read-sequence
    #+allegro excl:stream-read-sequence
-   #-(or lispworks sbcl openmcl cmu allegro)
-   (error "octet streams not supported in this implementation")))
+   #+abcl gray-streams:stream-read-sequence
+   #+(or ecl clisp) gray:stream-read-sequence
+   #-(or lispworks sbcl openmcl cmu allegro abcl ecl clisp)
+   (error 'ironclad-error :format-control "octet streams not supported in this implementation")))
 
 (defvar *stream-write-sequence-function*
   (quote
@@ -88,8 +104,10 @@
    #+openmcl ccl:stream-write-vector
    #+cmu ext:stream-write-sequence
    #+allegro excl:stream-write-sequence
-   #-(or lispworks sbcl openmcl cmu allegro)
-   (error "octet streams not supported in this implementation")))
+   #+abcl gray-streams:stream-write-sequence
+   #+(or ecl clisp) gray:stream-write-sequence
+   #-(or lispworks sbcl openmcl cmu allegro abcl ecl clisp)
+   (error 'ironclad-error :format-control "octet streams not supported in this implementation")))
 
 (defvar *stream-finish-output-function*
   (quote
@@ -98,8 +116,10 @@
    #+openmcl gray:stream-finish-output
    #+cmu ext:stream-finish-output
    #+allegro excl:stream-finish-output
-   #-(or lispworks sbcl openmcl cmu allegro)
-   (error "octet streams not supported in this implementation")))
+   #+abcl gray-streams:stream-finish-output
+   #+(or ecl clisp) gray:stream-finish-output
+   #-(or lispworks sbcl openmcl cmu allegro abcl ecl clisp)
+   (error 'ironclad-error :format-control "octet streams not supported in this implementation")))
 
 (defvar *stream-force-output-function*
   (quote
@@ -108,8 +128,10 @@
    #+openmcl gray:stream-force-output
    #+cmu ext:stream-force-output
    #+allegro excl:stream-force-output
-   #-(or lispworks sbcl openmcl cmu allegro)
-   (error "octet streams not supported in this implementation")))
+   #+abcl gray-streams:stream-force-output
+   #+(or ecl clisp) gray:stream-force-output
+   #-(or lispworks sbcl openmcl cmu allegro abcl ecl clisp)
+   (error 'ironclad-error :format-control "octet streams not supported in this implementation")))
 
 (defvar *stream-clear-output-function*
   (quote
@@ -118,8 +140,10 @@
    #+openmcl gray:stream-clear-output
    #+cmu ext:stream-clear-output
    #+allegro excl:stream-clear-output
-   #-(or lispworks sbcl openmcl cmu allegro)
-   (error "octet streams not supported in this implementation")))
+   #+abcl gray-streams:stream-clear-output
+   #+(or ecl clisp) gray:stream-clear-output
+   #-(or lispworks sbcl openmcl cmu allegro abcl ecl clisp)
+   (error 'ironclad-error :format-control "octet streams not supported in this implementation")))
 )
 
 
@@ -171,6 +195,30 @@
      (typecase seq
        (,type
         ,@body)
+       (t
+        (call-next-method))))
+  #+abcl
+  `(defmethod gray-streams:stream-read-sequence ((stream ,specializer) seq &optional (start 0) end)
+     (typecase seq
+       (,type
+        (let ((end (or end (length seq))))
+          ,@body))
+       (t
+        (call-next-method))))
+  #+ecl
+  `(defmethod gray:stream-read-sequence ((stream ,specializer) seq &optional (start 0) end)
+     (typecase seq
+       (,type
+        (let ((end (or end (length seq))))
+          ,@body))
+       (t
+        (call-next-method))))
+  #+clisp
+  `(defmethod gray:stream-read-sequence ((stream ,specializer) seq &key (start 0) end)
+     (typecase seq
+       (,type
+        (let ((end (or end (length seq))))
+          ,@body))
        (t
         (call-next-method)))))
 
@@ -219,16 +267,33 @@
      (typecase seq
        (,type
         ,@body)
+       (t
+        (call-next-method))))
+  #+abcl
+  `(defmethod gray-streams:stream-write-sequence ((stream ,specializer) seq &optional (start 0) end)
+     (typecase seq
+       (,type
+        (let ((end (or end (length seq))))
+          ,@body))
+       (t
+        (call-next-method))))
+  #+ecl
+  `(defmethod gray:stream-write-sequence ((stream ,specializer) seq &optional (start 0) end)
+     (typecase seq
+       (,type
+        (let ((end (or end (length seq))))
+          ,@body))
+       (t
+        (call-next-method))))
+  #+clisp
+  `(defmethod gray:stream-write-sequence ((stream ,specializer) seq &key (start 0) end)
+     (typecase seq
+       (,type
+        (let ((end (or end (length seq))))
+          ,@body))
        (t
         (call-next-method)))))
 
-(defun execute-with-digesting-stream (digest fn)
-  (with-open-stream (stream (make-digesting-stream digest))
-    (funcall fn stream)
-    (produce-digest stream)))
-
-(defmacro with-digesting-stream ((var digest) &body body)
-  `(execute-with-digesting-stream ,digest (lambda (,var) ,@body)))
 
 ;;; input streams
 
@@ -263,11 +328,15 @@
   "As MAKE-STRING-INPUT-STREAM, only with octets instead of characters."
   (declare (type simple-octet-vector buffer)
            (type index start)
-           (type (or index cl:null) end))
+           (type (or index null) end))
   (let ((end (or end (length buffer))))
     (make-instance 'octet-input-stream
                    :buffer buffer :index start :end end)))
 
+(defmacro with-octet-input-stream ((var buffer &optional (start 0) end) &body body)
+  `(with-open-stream (,var (make-octet-input-stream ,buffer ,start ,end))
+     ,@body))
+
 
 ;;; output streams
 
@@ -325,6 +394,11 @@ of a string output-stream."
   (make-instance 'octet-output-stream
                  :buffer (make-array 128 :element-type '(unsigned-byte 8))))
 
+(defmacro with-octet-output-stream ((var) &body body)
+  `(with-open-stream (,var (make-octet-output-stream))
+     ,@body
+     (get-output-stream-octets ,var)))
+
 
 ;;; digesting streams
 
@@ -338,8 +412,8 @@ of a string output-stream."
 (defmethod #.*stream-element-type-function* ((stream digesting-stream))
   '(unsigned-byte 8))
 
-(defun make-digesting-stream (digest)
-  (make-instance 'digesting-stream :digest (make-digest digest)))
+(defun make-digesting-stream (digest &rest args)
+  (make-instance 'digesting-stream :digest (apply #'make-digest digest args)))
 
 (defmethod #.*stream-write-byte-function* ((stream digesting-stream) byte)
   (declare (type (unsigned-byte 8) byte))
@@ -371,8 +445,17 @@ of a string output-stream."
     (unless (zerop position)
       (update-digest %digest buffer :start 0 :end position)
       (setf position 0))
-    (let ((copy (copy-digest %digest)))
-      (produce-digest copy :digest digest :digest-start digest-start))))
+    (produce-digest %digest :digest digest :digest-start digest-start)))
+
+(defun execute-with-digesting-stream (digest fn)
+  (with-open-stream (stream (make-digesting-stream digest))
+    (funcall fn stream)
+    (produce-digest stream)))
+
+(defmacro with-digesting-stream ((var digest &rest args) &body body)
+  `(with-open-stream (,var (make-digesting-stream ,digest ,@args))
+     ,@body
+     (produce-digest ,var)))
 
 
 ;;; encrypting and decrypting streams
@@ -400,9 +483,14 @@ of a string output-stream."
                                (direction :output) (padding :none))
   (declare (type stream-direction direction))
   (declare (ignorable padding))
+  (case mode
+    ((ctr :ctr) t)
+    ((stream :stream) t)
+    (t (error 'ironclad-error
+              :format-control "Encrypting streams support only CTR and STREAM modes")))
   (let* ((context (make-cipher cipher :mode mode :key key
                                :initialization-vector initialization-vector))
-         (block-length (max (block-length cipher) 16))
+         (block-length (max (block-length cipher) 4096))
          (buffer (make-array block-length :element-type '(unsigned-byte 8))))
     (if (eq direction :input)
         (make-instance 'encrypting-input-stream :stream stream
@@ -415,9 +503,14 @@ of a string output-stream."
                                (direction :input))
   (declare (type stream-direction direction))
   (declare (ignorable padding))
+  (case mode
+    ((ctr :ctr) t)
+    ((stream :stream) t)
+    (t (error 'ironclad-error
+              :format-control "Decrypting streams support only CTR and STREAM modes")))
   (let* ((context (make-cipher cipher :mode mode :key key
                                :initialization-vector initialization-vector))
-         (block-length (max (block-length cipher) 16))
+         (block-length (max (block-length cipher) 4096))
          (buffer (make-array block-length :element-type '(unsigned-byte 8))))
     (if (eq direction :input)
         (make-instance 'decrypting-input-stream :stream stream
@@ -425,39 +518,167 @@ of a string output-stream."
         (make-instance 'decrypting-output-stream :stream stream
                        :cipher context :buffer buffer))))
 
-;;; FIXME: padding issues
-(defun crypting-stream-read-byte (stream cryptfun)
+(defmethod #.*stream-read-byte-function* ((stream encrypting-input-stream))
   (with-slots (wrapped-stream cipher buffer n-bytes-valid position)
       stream
     (when (= position n-bytes-valid)
       (setf n-bytes-valid (read-sequence buffer wrapped-stream)
             position 0)
       (when (zerop n-bytes-valid)
-        (return-from crypting-stream-read-byte :eof))
-      (funcall cryptfun cipher buffer buffer))
+        (return-from #.*stream-read-byte-function* :eof))
+      (encrypt cipher buffer buffer :plaintext-end n-bytes-valid))
     (prog1 (aref buffer position)
       (incf position))))
 
-(defun crypting-stream-write-byte (stream byte cryptfun)
+(defmethod #.*stream-read-byte-function* ((stream decrypting-input-stream))
   (with-slots (wrapped-stream cipher buffer n-bytes-valid position)
       stream
     (when (= position n-bytes-valid)
-      (funcall cryptfun cipher buffer buffer)
-      (write-sequence buffer wrapped-stream)
-      (setf position 0))
-    (prog1 (setf (aref buffer position) byte)
+      (setf n-bytes-valid (read-sequence buffer wrapped-stream)
+            position 0)
+      (when (zerop n-bytes-valid)
+        (return-from #.*stream-read-byte-function* :eof))
+      (decrypt cipher buffer buffer :ciphertext-end n-bytes-valid))
+    (prog1 (aref buffer position)
       (incf position))))
 
-(defmethod #.*stream-read-byte-function* ((stream encrypting-input-stream))
-  (crypting-stream-read-byte stream #'encrypt))
-
-(defmethod #.*stream-read-byte-function* ((stream decrypting-input-stream))
-  (crypting-stream-read-byte stream #'decrypt))
-
 (defmethod #.*stream-write-byte-function* ((stream encrypting-output-stream) byte)
   (declare (type (unsigned-byte 8) byte))
-  (crypting-stream-write-byte stream byte #'encrypt))
+  (with-slots (wrapped-stream cipher buffer)
+      stream
+    (setf (aref buffer 0) byte)
+    (encrypt cipher buffer buffer :plaintext-end 1)
+    (write-byte (aref buffer 0) wrapped-stream)
+    byte))
 
 (defmethod #.*stream-write-byte-function* ((stream decrypting-output-stream) byte)
   (declare (type (unsigned-byte 8) byte))
-  (crypting-stream-write-byte stream byte #'decrypt))
+  (with-slots (wrapped-stream cipher buffer)
+      stream
+    (setf (aref buffer 0) byte)
+    (decrypt cipher buffer buffer :ciphertext-end 1)
+    (write-byte (aref buffer 0) wrapped-stream)
+    byte))
+
+(define-stream-read-sequence encrypting-input-stream simple-octet-vector
+  (with-slots (wrapped-stream cipher buffer n-bytes-valid position)
+      stream
+    (do ((n 0))
+        ((= start end) start)
+      (when (= position n-bytes-valid)
+        (setf n-bytes-valid (read-sequence buffer wrapped-stream)
+              position 0)
+        (when (zerop n-bytes-valid)
+          (return start))
+        (encrypt cipher buffer buffer :plaintext-end n-bytes-valid))
+      (setf n (min (- end start) (- n-bytes-valid position)))
+      (replace seq buffer :start1 start :end1 end :start2 position :end2 n-bytes-valid)
+      (incf start n)
+      (incf position n))))
+
+(define-stream-read-sequence decrypting-input-stream simple-octet-vector
+  (with-slots (wrapped-stream cipher buffer n-bytes-valid position)
+      stream
+    (do ((n 0))
+        ((= start end) start)
+      (when (= position n-bytes-valid)
+        (setf n-bytes-valid (read-sequence buffer wrapped-stream)
+              position 0)
+        (when (zerop n-bytes-valid)
+          (return start))
+        (decrypt cipher buffer buffer :ciphertext-end n-bytes-valid))
+      (setf n (min (- end start) (- n-bytes-valid position)))
+      (replace seq buffer :start1 start :end1 end :start2 position :end2 n-bytes-valid)
+      (incf start n)
+      (incf position n))))
+
+(define-stream-write-sequence encrypting-output-stream simple-octet-vector
+  (with-slots (wrapped-stream cipher buffer)
+      stream
+    (do ((buffer-length (length buffer))
+         (length (- end start))
+         (n 0))
+        ((zerop length))
+      (setf n (min buffer-length length))
+      (encrypt cipher seq buffer :plaintext-start start :plaintext-end (+ start n))
+      (write-sequence buffer wrapped-stream :end n)
+      (decf length n)
+      (incf start n))
+    seq))
+
+(define-stream-write-sequence decrypting-output-stream simple-octet-vector
+  (with-slots (wrapped-stream cipher buffer)
+      stream
+    (do ((buffer-length (length buffer))
+         (length (- end start))
+         (n 0))
+        ((zerop length))
+      (setf n (min buffer-length length))
+      (decrypt cipher seq buffer :ciphertext-start start :ciphertext-end (+ start n))
+      (write-sequence buffer wrapped-stream :end n)
+      (decf length n)
+      (incf start n))
+    seq))
+
+
+(defmacro with-encrypting-stream ((var stream cipher mode key
+                                   &key initialization-vector
+                                     (direction :output) (padding :none))
+                                  &body body)
+  `(with-open-stream (,var (make-encrypting-stream ,stream ,cipher ,mode ,key
+                                                   :initialization-vector ,initialization-vector
+                                                   :direction ,direction
+                                                   :padding ,padding))
+     ,@body))
+
+(defmacro with-decrypting-stream ((var stream cipher mode key
+                                   &key initialization-vector
+                                     (direction :input) (padding :none))
+                                  &body body)
+  `(with-open-stream (,var (make-decrypting-stream ,stream ,cipher ,mode ,key
+                                                   :initialization-vector ,initialization-vector
+                                                   :direction ,direction
+                                                   :padding ,padding))
+     ,@body))
+
+
+;;; authenticating streams
+
+(defclass authenticating-stream (#.*binary-output-stream-class*)
+  ((mac :initarg :mac :reader stream-mac)
+   (buffer :initform (make-array 64 :element-type '(unsigned-byte 8)) :reader stream-buffer)
+   (position :initform 0 :reader stream-buffer-position)))
+
+(defmethod #.*stream-element-type-function* ((stream authenticating-stream))
+  '(unsigned-byte 8))
+
+(defun make-authenticating-stream (mac key &rest args)
+  (make-instance 'authenticating-stream :mac (apply #'make-mac mac key args)))
+
+(defmethod #.*stream-write-byte-function* ((stream authenticating-stream) byte)
+  (declare (type (unsigned-byte 8) byte))
+  (with-slots (mac buffer position) stream
+    (setf (aref buffer position) byte)
+    (when (= (incf position) 64)
+      (update-mac mac buffer :start 0 :end 64)
+      (setf position 0))
+    byte))
+
+(define-stream-write-sequence authenticating-stream simple-octet-vector
+  (unless (zerop (stream-buffer-position stream))
+    (update-mac (stream-mac stream) (stream-buffer stream) :end (stream-buffer-position stream))
+    (setf (slot-value stream 'position) 0))
+  (update-mac (stream-mac stream) seq :start start :end end)
+  seq)
+
+(defmethod produce-mac ((stream authenticating-stream) &key digest (digest-start 0))
+  (with-slots (mac buffer position) stream
+    (unless (zerop position)
+      (update-mac mac buffer :start 0 :end position)
+      (setf position 0))
+    (produce-mac mac :digest digest :digest-start digest-start)))
+
+(defmacro with-authenticating-stream ((var mac key &rest args) &body body)
+  `(with-open-stream (,var (make-authenticating-stream ,mac ,key ,@args))
+     ,@body
+     (produce-mac ,var)))
